@@ -80,6 +80,7 @@ static impl_POA_Bonobo_ObjectDirectory *main_dir = NULL;
 static void
 od_dump_list (impl_POA_Bonobo_ObjectDirectory * od)
 {
+#if 0
 	int i, j, k;
 
 	for (i = 0; i < od->attr_servers._length; i++) {
@@ -127,6 +128,7 @@ od_dump_list (impl_POA_Bonobo_ObjectDirectory * od)
 			}
 		}
 	}
+#endif
 }
 #endif
 
@@ -167,24 +169,19 @@ update_registry (impl_POA_Bonobo_ObjectDirectory *servant, gboolean force_reload
         int i;
         time_t cur_time;
         gboolean must_load;
-        static int reload_recurse_depth = 0;
+        static gboolean doing_reload = FALSE;
 
-        
+        if (doing_reload)
+                return;
+        doing_reload = TRUE;
 
 #ifdef BONOBO_ACTIVATION_DEBUG
-        g_warning ("Update registry %d", reload_recurse_depth);
+        g_warning ("Update registry %p", servant->by_iid);
 #endif
 
-        if (reload_recurse_depth++)
-                return;
-
-        must_load = FALSE;
+        /* get first time init right */
+        must_load = (servant->by_iid == NULL);
         
-        /*
-         * Don't stat more than once every 5 seconds or activation
-         * could be too slow. This works even on the first read
-         * because then `time_did_stat is 0'
-         */
         cur_time = time (NULL);
 
         if (cur_time - 5 > servant->time_did_stat) {
@@ -192,9 +189,8 @@ update_registry (impl_POA_Bonobo_ObjectDirectory *servant, gboolean force_reload
                 
                 for (i = 0; servant->registry_source_directories[i] != NULL; i++) {
                         if (registry_directory_needs_update 
-                            (servant, servant->registry_source_directories[i])) {
+                            (servant, servant->registry_source_directories[i]))
                                 must_load = TRUE;
-                        }
                 }
         }
         
@@ -221,7 +217,7 @@ update_registry (impl_POA_Bonobo_ObjectDirectory *servant, gboolean force_reload
                         activation_clients_cache_notify ();
         }
 
-        reload_recurse_depth--;
+        doing_reload = FALSE;
 }
 
 static gchar **
@@ -468,7 +464,8 @@ quit_server_timeout (gpointer user_data)
 #endif
 
         if (!main_dir ||
-            g_hash_table_size (main_dir->active_servers) > RESIDUAL_SERVERS)
+            g_hash_table_size (main_dir->active_servers) > RESIDUAL_SERVERS ||
+            !activation_clients_is_empty_scan ())
                 g_warning ("Serious error handling server count, not quitting");
         else
                 g_main_loop_quit (main_loop);
@@ -478,15 +475,18 @@ quit_server_timeout (gpointer user_data)
         return FALSE;
 }
 
-static void
-check_quit (impl_POA_Bonobo_ObjectDirectory *servant)
+void
+check_quit (void)
 {
+        impl_POA_Bonobo_ObjectDirectory *servant = main_dir;
+
         /* We had some activity - so push out the shutdown timeout */
         if (servant->no_servers_timeout != 0)
                 g_source_remove (servant->no_servers_timeout);
         servant->no_servers_timeout = 0;
 
-        if (g_hash_table_size (servant->active_servers) <= RESIDUAL_SERVERS)
+        if (g_hash_table_size (servant->active_servers) <= RESIDUAL_SERVERS &&
+            activation_clients_is_empty_scan ())
                 servant->no_servers_timeout = g_timeout_add (
                         SERVER_IDLE_QUIT_TIMEOUT, quit_server_timeout, NULL);
 
@@ -531,9 +531,7 @@ active_server_cnx_broken (ORBitConnection *cnx,
                            servant->active_servers) - RESIDUAL_SERVERS);
 #endif
 
-        activation_clients_check ();
-
-        check_quit (servant);
+        check_quit ();
 }
 
 static void
@@ -562,7 +560,7 @@ add_active_server (impl_POA_Bonobo_ObjectDirectory *servant,
                               CORBA_Object_duplicate (object, NULL));
 
         if (cnx)
-                check_quit (servant);
+                check_quit ();
 }
 
 static void
@@ -578,7 +576,7 @@ remove_active_server (impl_POA_Bonobo_ObjectDirectory *servant,
 {
 	g_hash_table_remove (servant->active_servers, iid);
 
-        check_quit (servant);
+        check_quit ();
 }
 
 static Bonobo_RegistrationResult
@@ -809,6 +807,8 @@ bonobo_object_directory_re_check_fn (const char        *display,
 void
 bonobo_object_directory_reload (void)
 {
+#ifdef BONOBO_ACTIVATION_DEBUG
         g_print ("reloading our object directory!\n");
+#endif
         update_registry (main_dir, TRUE);
 }
